@@ -13,12 +13,13 @@
 # -period Under "angle" mode, the number of loops can also be controlled by argument ``period`` which controls the length
 #       of data a spiral loop corresponds to. Note in this case, argument ``end`` is ignored and the value for ``end`` is 
 #       internally recalculated.
+# -clockwise Whether the curve is in a closewise direction. If it is set to ``TRUE``, argument ``flip`` and ``reverse`` are ignored.
 # -flip How to flip the spiral? By default, the spiral starts from the origin of the coordinate and grows reverseclockwisely.
 #       The argument controls the growing direction of the spiral.
 # -reverse By default, the most inside of the spiral corresponds to the lower boundary of x-location. Setting the value to ``FALSE``
 #        can reverse the direction.
 # -polar_lines Whether draw the polar guiding lines.
-# -polar_lines_by Increment of the polar lines. Measured in degree.
+# -polar_lines_by Increment of the polar lines. Measured in degree. The value can also be a vector that defines where to add polar lines.
 # -polar_lines_gp Graphics parameters for the polar lines.
 # -padding Padding of the plotting region. The value can be a `grid::unit` of length of one to two.
 # -newpage Whether to apply `grid::grid.newpage` before making the plot?
@@ -51,8 +52,8 @@
 # make_plot("curve_length")
 spiral_initialize = function(xlim = c(0, 1), start = 360, end = 360*5, 
 	scale_by = c("angle", "curve_length"), period = NULL,
-	flip = c("none", "vertical", "horizontal", "both"), reverse = FALSE,
-	polar_lines = scale_by == "angle", polar_lines_by = 30, 
+	clockwise = FALSE, flip = c("none", "vertical", "horizontal", "both"), 
+	reverse = FALSE, polar_lines = scale_by == "angle", polar_lines_by = 30, 
 	polar_lines_gp = gpar(col = "#808080", lty = 3), 
 	padding = unit(5, "mm"), newpage = TRUE, vp_param = list()) {
 
@@ -84,6 +85,14 @@ spiral_initialize = function(xlim = c(0, 1), start = 360, end = 360*5,
 		end = xrange/period*360 + start
 	}
 
+	if(clockwise) {
+		rg = end - start
+		start = 180 - start %% 360 + floor(start / 360)*360
+		end = start + rg
+		flip = "horizontal"
+		reverse = FALSE
+	}
+
 	spiral = create_spiral(start = start, end = end, xlim = xlim, scale_by = scale_by, flip = flip, reverse = reverse)
 	spiral_env$spiral = spiral
 	spiral_env$i_spiral = spiral_env$i_spiral + 1
@@ -123,7 +132,11 @@ spiral_initialize = function(xlim = c(0, 1), start = 360, end = 360*5,
 	pushViewport(do.call(viewport, vp_param2))
 
 	if(polar_lines) {
-		d = seq(0, 360, by = polar_lines_by)
+		if(length(polar_lines_by) == 1) {
+			d = seq(0, 360, by = polar_lines_by)
+		} else {
+			d = polar_lines_by
+		}
 		if(d[length(d)] == 360) d = d[-length(d)]
 		dm = matrix(nrow = length(d), ncol = 4)
 		for(i in seq_along(d)) {
@@ -143,8 +156,10 @@ spiral_initialize = function(xlim = c(0, 1), start = 360, end = 360*5,
 # -end End of the spiral, in degrees. By default it is automatically calculated.
 # -unit_on_axis Units on the axis.
 # -period Which period to use?
+# -normalize_year Whether to enforce one loop to represent a complete year?
 # -period_per_loop How many periods to put in a loop?
 # -polar_lines_by By default different value of ``polar_lines_by`` is set for different ``period``. E.g. 360/7 is set if ``period`` is "weeks" or 360/24 is set if ``peroid`` is set to "hours".
+#              When ``period`` is year and ``unit_on_axis`` is day, the proportion of sectors by polar lines corresponds to the proportion of month days in a year.
 # -verbose Whether to print messages?
 # -... All pass to `spiral_initialize`.
 #
@@ -170,8 +185,9 @@ spiral_initialize = function(xlim = c(0, 1), start = 360, end = 360*5,
 # spiral_axis()
 spiral_initialize_by_time = function(xlim, start = NULL, end = NULL,
 	unit_on_axis = c("days", "months", "weeks", "hours", "mins", "secs"), 
-	period = c("years", "months", "weeks", "days", "hours", "mins"),
-	period_per_loop = 1, polar_lines_by = NULL, verbose = TRUE, ...) {
+	period = c("years", "months", "weeks", "days", "hours", "mins"), 
+	normalize_year = FALSE, period_per_loop = 1, polar_lines_by = NULL, 
+	verbose = TRUE, ...) {
 
 	xlim_is_date = inherits(xlim, "Date")
 	xlim = as.POSIXlt(xlim)
@@ -246,9 +262,20 @@ spiral_initialize_by_time = function(xlim, start = NULL, end = NULL,
 		}
 	}
 
+	if(verbose) {
+		if(period == "years" && unit_on_axis == "days" && !normalize_year) {
+			message_wrap("When the period is year and the unit on axis is day, a loop can only represent 364 days (52 weeks) under default settings, which helps to correspond weekdays between years, but 1 or 2 days from the current year will be moved and accumulated to the next year. You can set argument `normalize_year = TRUE` to enforce every loop to represent a complete year, but note you might not be able to perfectly correspond weekdays between different years. Set argument `verbose = FALSE` to turn off this message.")
+		}
+	}
+
 	if(is.null(polar_lines_by)) {
 		if(period == "years") {
-			polar_lines_by = (360/12)/period_per_loop
+			if(normalize_year && unit_on_axis == "days") {
+				polar_lines_by = cumsum(days_in_month(1:12))/365*360
+				polar_lines_by = c(0, polar_lines_by[-12])
+			} else {
+				polar_lines_by = (360/12)/period_per_loop
+			}
 		} else if(period == "weeks") {
 			polar_lines_by = (360/7)/period_per_loop
 		} else if(period == "days") {
@@ -261,54 +288,65 @@ spiral_initialize_by_time = function(xlim, start = NULL, end = NULL,
 	time_start = xlim[1]
 	time_end = xlim[2]
 
-	get_numeric_x = local({
+	get_x_from_data = local({
 		time_start = time_start
-		function(x) {
+		unit_on_axis = unit_on_axis
+		function(x, where = "center") {
 			if(is.numeric(x)) {
 				return(x)
 			}
 			x = as.POSIXlt(x)
-			time_difference(time_start, x, unit_on_axis)
+			v = time_difference(time_start, x, unit_on_axis)  # assume time_start with x = 0
+			if(where == "right") {
+				v + 0.5
+			} else if(where == "left") {
+				v - 0.5
+			} else {
+				v
+			}
 		}
 	})
 
-	get_data_x = local({
+	get_data_from_x = local({
 		time_start = time_start
+		unit_on_axis = unit_on_axis
 		function(x) {
+			x = x - current_spiral()$other$x_offset[1]
+			x = round(x)
 			add_time(time_start, x, unit_on_axis)
 		}
 	})
 
 	if(unit_on_axis == "days") {
-		get_character_x = local({
+		get_character_from_x = local({
 			time_start = time_start
 			function(x) {
-				as.character(time_start + days(x))
+				as.character(get_data_from_x(x))
 			}
 		})
 	} else if(unit_on_axis == "months") {
-		get_character_x = local({
+		get_character_from_x = local({
 			time_start = time_start
 			function(x) {
-				t = time_start %m+% months(x)
+				t = get_data_from_x(x)
 				format(t, "%Y-%m")
 			}
 		})
 	} else if(unit_on_axis == "weeks") {
-		get_character_x = local({
+		get_character_from_x = local({
 			time_start = time_start
 			function(x) {
-				t = time_start + weeks(x)
+				t = get_data_from_x(x)
 				format(t, "%Y-%m-%d")
 			}
 		})
 	} else if(unit_on_axis == "hours") {
-		get_character_x = local({
+		get_character_from_x = local({
 			time_start = time_start
 			in_a_same_year = year(time_end) == year(time_start)
 			in_a_same_day = day(time_end) == day(time_start)
 			function(x) {
-				t = time_start + hours(x)
+				t = get_data_from_x(x)
 				if(in_a_same_year && in_a_same_day) {
 					format(t, "%Hh")
 				} else if(in_a_same_year) {
@@ -319,12 +357,12 @@ spiral_initialize_by_time = function(xlim, start = NULL, end = NULL,
 			}
 		})
 	} else if(unit_on_axis == "mins") {
-		get_character_x = local({
+		get_character_from_x = local({
 			time_start = time_start
 			in_a_same_year = year(time_end) == year(time_start)
 			in_a_same_day = day(time_end) == day(time_start)
 			function(x) {
-				t = time_start + minutes(x)
+				t = get_data_from_x(x)
 				if(in_a_same_year && in_a_same_day) {
 					format(t, "%H:%M")
 				} else if(in_a_same_year) {
@@ -335,12 +373,12 @@ spiral_initialize_by_time = function(xlim, start = NULL, end = NULL,
 			}
 		})
 	} else if(unit_on_axis == "secs") {
-		get_character_x = local({
+		get_character_from_x = local({
 			time_start = time_start
 			in_a_same_year = year(time_end) == year(time_start)
 			in_a_same_day = day(time_end) == day(time_start)
 			function(x) {
-				t = time_start + seconds(x)
+				t = get_data_from_x(x)
 				if(in_a_same_year && in_a_same_day) {
 					format(t, "%H:%M:%S")
 				} else if(in_a_same_year) {
@@ -352,75 +390,179 @@ spiral_initialize_by_time = function(xlim, start = NULL, end = NULL,
 		})
 	}
 
-	xlim = get_numeric_x(xlim)
+	## redefine 
+	if(normalize_year && period== "years" && unit_on_axis == "days") {
+		get_x_from_data = local({
+			time_start = time_start
+			function(x, where = "center") {
 
-	start_of_the_year = as.POSIXlt("2021-01-01")
-	year(start_of_the_year) = year(time_start)
+				if(is.numeric(x)) {
+					return(x)
+				}
+				x = as.POSIXlt(x)
+				y = year(x)
+
+				first_year = year(time_start)
+				first_day_in_year = as.POSIXlt(paste0(y, "-01-01"))
+				last_day_in_year = as.POSIXlt(paste0(y, "-12-31"))
+				days_in_year = calc_days_in_year(y)
+
+				v = (y - first_year)*360 + time_difference(first_day_in_year, x)/days_in_year*360
+				if(where == "left") {
+					v - 0.5/calc_days_in_year(y)*360
+				} else if(where == "right") {
+					v + 0.5/calc_days_in_year(y)*360
+				} else {
+					v
+				}
+			}
+		})
+
+		get_data_from_x = local({
+			time_start = time_start
+			function(x) {
+				x = x - current_spiral()$other$x_offset[1]
+				first_year = year(time_start)
+				which_loop = ceiling(x/360) - 1; #which_loop[x %% 360 == 0] = which_loop[x %% 360 == 0] + 1
+				first_day_in_year = as.POSIXlt(paste0(first_year + which_loop, "-01-01"))
+				last_day_in_year = as.POSIXlt(paste0(first_year + which_loop, "-12-31"))
+				days_in_year = calc_days_in_year(first_year + which_loop)
+
+				add_time(first_day_in_year, round((x %% 360)/360*days_in_year), "days")
+			}
+		})
+
+		get_character_from_x = local({
+			time_start = time_start
+			function(x) {
+				as.character(get_data_from_x(x))
+			}
+		})
+	}
+
+
+	get_start_of_the_year = function(t) {
+		month(t) = 1
+		day(t) = 1
+		t
+	}
+	get_start_of_the_month = function(t) {
+		day(t) = 1
+		hour(t) = 0
+		minute(t) = 0
+		second(t) = 0
+		t
+	}
+	get_start_of_the_day = function(t) {
+		hour(t) = 0
+		minute(t) = 0
+		second(t) = 0
+		t
+	}
+	get_start_of_the_hour = function(t) {
+		minute(t) = 0
+		second(t) = 0
+		t
+	}
+	get_start_of_the_minute = function(t) {
+		second(t) = 0
+		t
+	}
 
 	if(unit_on_axis == "months") {
-		if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/12/period_per_loop)*360
-		end = 360*(time_difference(time_start, time_end, unit_on_axis)/12/period_per_loop) + start
+		if(is.null(start)) start = (1 + time_difference(get_start_of_the_year(time_start), time_start, unit_on_axis)/12/period_per_loop)*360
+		end = 360*(time_range(time_start, time_end, unit_on_axis)/12/period_per_loop) + start
 	} else if(unit_on_axis == "weeks") {
-		if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/52/period_per_loop)*360
-		end = 360*(time_difference(time_start, time_end, unit_on_axis)/52/period_per_loop) + start
+		if(is.null(start)) start = (1 + time_difference(get_start_of_the_year(time_start), time_start, unit_on_axis)/52/period_per_loop)*360
+		end = 360*(time_range(time_start, time_end, unit_on_axis)/52/period_per_loop) + start
 	} else if(unit_on_axis == "days") {
 		if(period == "years") {
-			if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/(52*7)/period_per_loop)*360
-			end = 360*(time_difference(time_start, time_end, unit_on_axis)/(52*7)/period_per_loop) + start
+			if(is.null(start)) start = (1 + time_difference(get_start_of_the_year(time_start), time_start, unit_on_axis)/(52*7)/period_per_loop)*360
+			end = 360*(time_range(time_start, time_end, unit_on_axis)/(52*7)/period_per_loop) + start
 		} else if(period == "months") {
-			if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/30/period_per_loop)*360
-			end = 360*(time_difference(time_start, time_end, unit_on_axis)/30/period_per_loop) + start
+			if(is.null(start)) start = (1 + time_difference(get_start_of_the_month(time_start), time_start, unit_on_axis)/30/period_per_loop)*360
+			end = 360*(time_range(time_start, time_end, unit_on_axis)/30/period_per_loop) + start
 		} else if(period == "weeks") {
-			if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/7/period_per_loop)*360
-			end = 360*(time_difference(time_start, time_end, unit_on_axis)/7/period_per_loop) + start
+			if(is.null(start)) start = (1 + time_difference(get_start_of_the_month(time_start), time_start, unit_on_axis)/7/period_per_loop)*360
+			end = 360*(time_range(time_start, time_end, unit_on_axis)/7/period_per_loop) + start
 		} 
 	} else if(unit_on_axis == "hours") {
 		if(period == "months") {
-			if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/(30*24)/period_per_loop)*360
-			end = 360*(time_difference(time_start, time_end, unit_on_axis)/(30*24)/period_per_loop) + start
+			if(is.null(start)) start = (1 + time_difference(get_start_of_the_month(time_start), time_start, unit_on_axis)/(30*24)/period_per_loop)*360
+			end = 360*(time_range(time_start, time_end, unit_on_axis)/(30*24)/period_per_loop) + start
 		} else if(period == "weeks") {
-			if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/(7*24)/period_per_loop)*360
-			end = 360*(time_difference(time_start, time_end, unit_on_axis)/(7*24)/period_per_loop) + start
+			if(is.null(start)) start = (1 + time_difference(get_start_of_the_month(time_start), time_start, unit_on_axis)/(7*24)/period_per_loop)*360
+			end = 360*(time_range(time_start, time_end, unit_on_axis)/(7*24)/period_per_loop) + start
 		} else if(period == "days") {
-			if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/24/period_per_loop)*360
-			end = 360*(time_difference(time_start, time_end, unit_on_axis)/24/period_per_loop) + start
+			if(is.null(start)) start = (1 + time_difference(get_start_of_the_day(time_start), time_start, unit_on_axis)/24/period_per_loop)*360
+			end = 360*(time_range(time_start, time_end, unit_on_axis)/24/period_per_loop) + start
 		}
 	} else if(unit_on_axis == "mins") {
 		if(period == "days") {
-			if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/(24*60)/period_per_loop)*360
-			end = 360*(time_difference(time_start, time_end, unit_on_axis)/(24*60)/period_per_loop) + start
+			if(is.null(start)) start = (1 + time_difference(get_start_of_the_day(time_start), time_start, unit_on_axis)/(24*60)/period_per_loop)*360
+			end = 360*(time_range(time_start, time_end, unit_on_axis)/(24*60)/period_per_loop) + start
 		} else if(period == "hours") {
-			if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/60/period_per_loop)*360
-			end = 360*(time_difference(time_start, time_end, unit_on_axis)/60/period_per_loop) + start
+			if(is.null(start)) start = (1 + time_difference(get_start_of_the_hour(time_start), time_start, unit_on_axis)/60/period_per_loop)*360
+			end = 360*(time_range(time_start, time_end, unit_on_axis)/60/period_per_loop) + start
 		}
 	} else if(unit_on_axis == "secs") {
 		if(period == "hours") {
-			if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/(60*60)/period_per_loop)*360
-			end = 360*(time_difference(time_start, time_end, unit_on_axis)/(60*60)/period_per_loop) + start
+			if(is.null(start)) start = (1 + time_difference(get_start_of_the_hour(time_start), time_start, unit_on_axis)/(60*60)/period_per_loop)*360
+			end = 360*(time_range(time_start, time_end, unit_on_axis)/(60*60)/period_per_loop) + start
 		} else if(period == "mins") {
-			if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/60/period_per_loop)*360
-			end = 360*(time_difference(time_start, time_end, unit_on_axis)/60/period_per_loop) + start
+			if(is.null(start)) start = (1 + time_difference(get_start_of_the_minute(time_start), time_start, unit_on_axis)/60/period_per_loop)*360
+			end = 360*(time_range(time_start, time_end, unit_on_axis)/60/period_per_loop) + start
 		}
 	}
-	p = 1/(xlim[2] - xlim[1])
-	start = start - (end - start)*p/2
-	end = end + (end - start)*p/2
-	xlim[1] = xlim[1] - 0.5
-	xlim[2] = xlim[2] + 0.5
-	spiral_initialize(xlim = xlim, start = start, end = end, polar_lines_by = polar_lines_by, ...)
+
+	x_xlim = get_x_from_data(xlim)
+	x_xlim[1] = x_xlim[1] - 0.5
+	x_xlim[2] = x_xlim[2] + 0.5
+
+	x_offset = c(-0.5, 0.5)
+
+	if(normalize_year && period== "years" && unit_on_axis == "days") {
+
+		year_start = year(time_start)
+		year_end = year(time_end)
+
+		if(is.null(start)) {
+			start = time_difference(as.POSIXlt(paste0(year_start, "-01-01")), time_start, "days")/calc_days_in_year(year_start)*360 + 360
+		}
+
+		if(year_start == year_end) {
+			end = (1 + time_difference(time_start, time_end, "days"))/calc_days_in_year(year_start)*360 + start
+		} else {
+			end = start - time_difference(as.POSIXlt(paste0(year_start, "-01-01")), time_start, "days")/calc_days_in_year(year_start)*360 +
+				  (year(time_end) - year(time_start))*360 +
+			      (1 + time_difference(as.POSIXlt(paste0(year_end, "-01-01")), time_end, "days"))/calc_days_in_year(year_end)*360
+		}
+		x_xlim = get_x_from_data(xlim)
+		x_offset = c(- 0.5/calc_days_in_year(year(xlim[1]))*360, 0.5/calc_days_in_year(year(xlim[2]))*360)
+		x_xlim = x_xlim + x_offset
+	}
+
+	spiral_initialize(xlim = x_xlim, start = start, end = end, polar_lines_by = polar_lines_by, ...)
 
 	spiral = current_spiral()
 	spiral$xclass = "Time"
-	spiral$get_numeric_x = get_numeric_x
-	spiral$get_character_x = get_character_x
+	spiral$get_x_from_data = get_x_from_data
+	spiral$get_character_from_x = get_character_from_x
+	spiral$get_data_from_x = get_data_from_x
 
 	spiral$other = list(unit_on_axis = unit_on_axis, 
-		                period = period)
+		                period = period,
+		                normalize_year = normalize_year,
+		                x_offset = x_offset)
 
 	invisible(NULL)
 }
 
 
+calc_days_in_year = function(y) {
+	time_difference(as.POSIXlt(paste0(y, "-01-01")), as.POSIXlt(paste0(y, "-12-31"))) + 1
+}
+
 
 # unit: months, weeks, days, hours, mins, secs
 time_difference = function(x, y, unit = "days") {
@@ -450,6 +592,10 @@ time_difference = function(x, y, unit = "days") {
 	}
 }
 
+time_range = function(...) {
+	time_difference(...) + 1
+}
+
 add_time = function(t, diff, unit) {
 	if(unit == "days") {
 		t + lubridate::days(diff)
@@ -491,7 +637,7 @@ add_time = function(t, diff, unit) {
 # spiral_axis()
 spiral_initialize_by_gcoor = function(xlim, scale_by = "curve_length", ...) {
 
-	get_character_x = function(x) {
+	get_character_from_x = function(x) {
 		x2 = character(length(x))
 		l1 = x >= 1e6
 		x2[l1] = paste(x[l1]/1000000, "MB", sep = "")
@@ -506,13 +652,14 @@ spiral_initialize_by_gcoor = function(xlim, scale_by = "curve_length", ...) {
 			last_unit = "KB"
 		}
 		x2[l3] = paste(x[l3], last_unit, sep = "")
+		x2
 	}
 
 	spiral_initialize(xlim = xlim, scale_by = scale_by, ...)
 
 	spiral = current_spiral()
 	spiral$xclass = "Genomic positions"
-	spiral$get_character_x = get_character_x
+	spiral$get_character_from_x = get_character_from_x
 
 	invisible(NULL)
 }
